<?php

namespace Drupal\dopup\Controller;

use Drupal\Component\Utility\Unicode;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Returns responses for dopup routes.
 */
class DopupAutoCompleteController extends ControllerBase {


  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Builds the response.
   */
  public function build() {
    $webforms = $this->entityTypeManager->getStorage('webform')
      ->getQuery()
      ->condition('category', 'dopup')
      ->execute();
    $webforms = array_map([$this, 'mapValueLabel'], array_values($webforms));
    return new JsonResponse($webforms);
  }

  /**
   * Map webform IDs array.
   *
   * Function to map webform ids to readable titles.
   *
   * @param array $webform
   *   Webform ID.
   *
   * @return array
   *   An array of value and label.
   */
  protected function mapValueLabel($webform) {
    return ['value' => $webform, 'label' => str_replace('_', ' ', Unicode::ucfirst($webform))];
  }

}
